from __future__ import annotations

import time
import datetime


def convert_datetime_to_timestamp(target_date:datetime.datetime | datetime.date):
    """
    datetime.datetime 혹은 datetime.date 를 타임스탬프로 변환
    """

    assert isinstance(target_date, datetime.datetime) or isinstance(target_date, datetime.date), "잘못된 타입 입력"

    if isinstance(target_date, datetime.datetime):
        timestamp = int(time.mktime(datetime.datetime(target_date.year, target_date.month, target_date.day, target_date.hour, target_date.minute, target_date.second).timetuple()))

    elif isinstance(target_date, datetime.date):
        timestamp = int(time.mktime(datetime.datetime(target_date.year, target_date.month, target_date.day, 9, 0, 0).timetuple()))

    return timestamp



import datetime


def first_day_of_month(any_day:datetime.date) -> datetime.date:    
    return any_day.replace(day=1)


def last_day_of_month(any_day:datetime.date) -> datetime.date:
    # this will never fail
    # get close to the end of the month for any day, and add 4 days 'over'
    next_month = any_day.replace(day=28) + datetime.timedelta(days=4)
    # subtract the number of remaining 'overage' days to get last day of current month, or said programattically said, the previous day of the first of next month
    return next_month - datetime.timedelta(days=next_month.day)


def first_day_of_week(any_day:datetime.date) -> datetime.date:
    return any_day - datetime.timedelta(days=any_day.weekday())


def first_day_of_last_month(any_day:datetime.date=None) -> datetime.date:
    if any_day is None:
        any_day = datetime.date.today()

    return (any_day.replace(day=1) - datetime.timedelta(days=1)).replace(day=1)


def last_day_of_last_month(any_day:datetime.date=None) -> datetime.date:
    if any_day is None:
        any_day = datetime.date.today()

    return any_day.replace(day=1) - datetime.timedelta(days=1)


def last_day_of_last_week(any_day:datetime.date=None) -> datetime.date:
    if any_day is None:
        any_day = datetime.date.today()
    
    return any_day - datetime.timedelta(days=any_day.weekday()+1)  # 이전 주 일요일


import re

def convert_hannum_to_num(hannum, is_int=False):
    """
    한글 형태로 표시된 숫자 아라비아 숫자로 변환
    """

    num_only_regex = re.compile("[\d.\-\+]")
    result_num = num_only_regex.findall(hannum)
    result_num = "".join(result_num)

    not_num_regex = re.compile("[십백천만억]")
    digit = not_num_regex.findall(hannum)
    digit = "".join(digit)

    num_digit = 1
    for d in digit:
        if d == "십":
            num_digit *= 10
        elif d == "백":
            num_digit *= 100
        elif d == "천":
            num_digit *= 1000
        elif d == "만":
            num_digit *= 10000
        elif d == "억":
            num_digit *= 100000000

    if result_num in ["", "-"]:
        return ""

    if is_int:
        return int(num_digit*float(result_num))
    else:
        return num_digit*float(result_num)


def convert_engnum_to_num(hannum, is_int=False):
    """
    영어 형태로 표시된 숫자 아라비아 숫자로 변환
    """

    num_only_regex = re.compile("[\d.\-\+]")
    result_num = num_only_regex.findall(hannum)
    result_num = "".join(result_num)

    not_num_regex = re.compile("[MK]")
    digit = not_num_regex.findall(hannum)
    digit = "".join(digit)

    if digit == "M":
        digit = 1000000
    elif digit == "K":
        digit = 1000
    else:
        digit = 1

    if result_num in ["", "-"]:
        return ""

    if is_int:
        return int(digit*float(result_num))
    else:
        return digit*float(result_num)